//
//  EMBEmberWebSnappingWindowController.h
//  EmberMac
//
//  Created by Damien DeVille on 1/12/14.
//  Copyright (c) 2014 Realmac Software. All rights reserved.
//

#import <Cocoa/Cocoa.h>

@class EMBEmberLibrary;

@interface EMBSnappingMetadataInputWindowController : NSWindowController

@property (strong, nonatomic) EMBEmberLibrary *emberLibrary;
@property (strong, nonatomic) NSManagedObjectContext *managedObjectContext;

/*!
	\brief
	Calls `prepareAndShowWithMetadata:presentationFrame:screen:completion:` with `nil` for `buttonTitle`, YES for `showBrowserOption`, `CGRectNull` for `presentationFrame` and `[NSScreen mainScreen]` for `screen`
 */
- (void)prepareAndShowWithMetadata:(NSDictionary *)metadata completion:(void (^)(BOOL confirmed, NSDictionary *updatedMetadata))completionHandler;

/*!
	\brief
	Prepare the window by populating its controls from the metadata. Invoke the completion block when completed, either when confirming or dismissing
	
	\param metadata
	The snap metadata used to populate the initial value of the controls
	
	\param buttonTitle
	The title displayed for the confirmation button. By default `Snap`
	
	\param showBrowserOption
	Whether the Open in Ember Browser button should be visible
	
	\param presentationFrame
	The frame in which to present the window. The window will be centered in this frame
	
	\param screen
	The screen on which the window will be presented. Cannot be nil
	
	\param completion
	A completion block invoked when the confirmation button is clicked or whenever the window is dismissed. The `confirmed` parameter indicates wether it was cancelled or confirmed
 */
- (void)prepareAndShowWithMetadata:(NSDictionary *)metadata buttonTitle:(NSString *)buttonTitle showBrowserOption:(BOOL)showBrowserOption presentationFrame:(CGRect)presentationFrame screen:(NSScreen *)screen completion:(void (^)(BOOL confirmed, NSDictionary *updatedMetadata))completionHandler;

@end
