//
// Copyright (C) 2008-2012 Realmac Software Ltd
//
// These coded instructions, statements, and computer programs contain
// unpublished proprietary information of Realmac Software Ltd
// and are protected by copyright law. They may not be disclosed
// to third parties or copied or duplicated in any form, in whole or
// in part, without the prior written consent of Realmac Software Ltd.
//

#import <Foundation/Foundation.h>

/*!
	\brief
	This class represents an abstract concurrent operation
 */
@interface EMBConcurrentOperation : NSOperation

/*!
	\brief
	Designated initializer
 */
- (id)init;

/*!
	\brief
	Support creating a concurrent operation with a given work queue
 */
- (id)initWithWorkQueue:(NSOperationQueue *)workQueue;

/*!
	\brief
	The block to add a work operation
 */
typedef void (^EMBConcurrentOperationAddWorkBlock)(NSOperation *operation);
/*!
	\brief
	Override this in the subclass to provide work operations to be executed
 */
- (void)setupOperations:(EMBConcurrentOperationAddWorkBlock)addWorkOperation;

/*!
	\brief
	Override this in the subclass to do cancellation work. This will be invoked from the `cancel` method in a concurrency-safe manner.
 */
- (void)cancelWork;

/*!
	\brief
	Cleanup is called when the operation goes isFinished
 */
- (void)cleanup;

@end
