//
//  EMBEmberTask.h
//  EmberCommon
//
//  Created by Damien DeVille on 09/11/2012.
//  Copyright (c) 2012 Realmac Software. All rights reserved.
//

#import <Foundation/Foundation.h>

#import "EMBEmberCoreConcurrency/EMBEmberCoreConcurrency.h"

@class EMBEmberLibrary;
@class NSManagedObjectContext;

@interface EMBEmberTask : EMBConcurrentOperation <EMBProgressOperation>

/*!
	\brief
	Create a task that will enqueue the work operation and save the managed object context.
	
	It is a programmer error to pass a workOperation that has already been added to a queue (or add it to a queue after initialization).
 */
- (id)initWithWorkOperation:(NSOperation <EMBProgressOperation> *)workOperation managedObjectContext:(NSManagedObjectContext *)managedObjectContext emberLibrary:(EMBEmberLibrary *)emberLibrary;

@property (readonly, copy, atomic) id (^completionProvider)(NSError **errorRef);

@property (strong, nonatomic) NSArray *whiteListErrors;

@property (copy, nonatomic) NSError * (^errorRefiner)(NSError *error);

@end

/*
	These notifications are posted inside the managed object context's performBlock,
	wrapping the save invocation.
 */
extern NSString * const EMBEmberTaskWillSaveNotification;
extern NSString * const EMBEmberTaskDidSaveNotification;
