//
//  RMPopoverBackgroundView.h
//  Analog
//
//  Created by Keith Duncan on 08/03/2011.
//  Copyright 2011 Realmac Software. All rights reserved.
//

#import <Cocoa/Cocoa.h>

@interface RMPopoverBackgroundView : NSView

/*!
	\brief
	contentInset is the distance from the inner edge of an image to the content placement, can be -ve, must be the same for all sides
	frameInset is the distance from the inner edge of an image to the popover 
 */
struct RMPopoverBackgroundViewStyleMetrics {
	CGFloat bodyContentInset;
	CGFloat bodyFrameInset;
	
	CGFloat headerContentHeightInsets[2];
	CGFloat headerFrameInset;
	
	CGPoint arrowImageTipPosition; // coordinate (in arrow-image coordinates) of the arrow tip
};
typedef struct RMPopoverBackgroundViewStyleMetrics RMPopoverBackgroundViewStyleMetrics;

/*!
	\brief
	Image lookup uses the methods in `NSBundle(RMAdditions)`
	
	\return
	A zero sized view, the frame MUST be set prior to drawing.
 */
- (id)initWithMetrics:(RMPopoverBackgroundViewStyleMetrics)metrics imageBundle:(NSBundle *)imageBundle imageStyle:(NSString *)imageStyle;

@property (readonly, assign, nonatomic) RMPopoverBackgroundViewStyleMetrics metrics;
@property (readonly, retain, nonatomic) NSBundle *imageBundle;
@property (readonly, copy, nonatomic) NSString *imageStyle;

/*!
	\brief
	Calculate the required frame size for a content view's size.
 */
- (CGSize)calculateFrameSizeForContentSize:(CGSize)contentSize;

/*!
	\brief
	When YES a header is drawn above the content rect in the initialised style.
 */
@property (assign, nonatomic) BOOL drawsHeader;

/*!
	\brief
	Calculate the header bounds when the receiver's bounds are equal to the argument.
 */
- (CGRect)calculateHeaderRectForBounds:(CGRect)bounds;
/*!
	\brief
	The header rect is available in all instances irrespective of the value of `showsHeader`
	Though, the header background is only shown if `showsHeader` is set to YES.
 */
@property (readonly, nonatomic) CGRect headerRect;

/*!
	\brief
	Calculate the body bounds when the receiver's bounds are are equal to the argument.
 */
- (CGRect)calculateBodyRectForBounds:(CGRect)bounds;
/*!
	\brief
	Once the frame has been set, this returns the layout rect for the popover content.
	
	\return
	A rect in the coordinate system of the receiver.
 */
@property (readonly, nonatomic) CGRect bodyRect;

/*!
	\brief
	An edge may be unsupported if there's not enough space for the arrow image.
 */
- (BOOL)calculateSupportedArrowEdge:(CGRectEdge)arrowEdge forBounds:(CGRect)bounds;
/*!
	\brief
	All instances support all orientation without changing the content rect.
 */
@property (assign, nonatomic) CGRectEdge arrowEdge;

/*!
	\brief
	Used to determine where in the arrowEdge dimention the arrow is drawn
 */
@property (assign, nonatomic) CGFloat arrowOffset;

/*!
	\brief
	The arrow tip will lies somewhere on this rect, dependent on `showsHeader`.
 */
- (CGRect)calculateAlignmentRectForContentSize:(CGSize)contentSize;

@end
